from datadog import DogStatsd, initialize, statsd

from dagster import Field, resource


class DataDogResource:
    '''DataDogResource

    This resource is a thin wrapper over the dogstatsd library:

    https://datadogpy.readthedocs.io/en/latest/#datadog-dogstatsd-module

    As such, we directly mirror the public API methods of DogStatsd here; you can refer to the
    DataDog documentation above for how to use this resource.
    '''

    # Mirroring levels from the dogstatsd library
    OK, WARNING, CRITICAL, UNKNOWN = (
        DogStatsd.OK,
        DogStatsd.WARNING,
        DogStatsd.CRITICAL,
        DogStatsd.UNKNOWN,
    )

    def __init__(self, api_key, app_key):
        initialize(api_key=api_key, app_key=app_key)

        # Pull in methods from the dogstatsd library
        for method in [
            'event',
            'gauge',
            'increment',
            'decrement',
            'histogram',
            'distribution',
            'set',
            'service_check',
            'timed',
            'timing',
        ]:
            setattr(self, method, getattr(statsd, method))


@resource(
    {
        'api_key': Field(str, description='Datadog API key'),
        'app_key': Field(str, description='Datadog application key'),
    },
    description='This resource is for publishing to DataDog',
)
def datadog_resource(context):
    return DataDogResource(
        context.resource_config.get('api_key'), context.resource_config.get('app_key')
    )



import argparse
import sys

from setuptools import find_packages, setup


def get_version(name):
    version = {}
    with open('dagster_datadog/version.py') as fp:
        exec(fp.read(), version)  # pylint: disable=W0122

    if name == 'dagster-datadog':
        return version['__version__']
    elif name == 'dagster-datadog-nightly':
        return version['__nightly__']
    else:
        raise Exception('Shouldn\'t be here: bad package name {name}'.format(name=name))


parser = argparse.ArgumentParser()
parser.add_argument('--nightly', action='store_true')


def _do_setup(name='dagster-datadog'):
    setup(
        name='dagster_datadog',
        version=get_version(name),
        author='Elementl',
        license='Apache-2.0',
        description='Package for datadog Dagster framework components.',
        url='https://github.com/dagster-io/dagster/tree/master/python_modules/libraries/dagster-datadog',
        classifiers=[
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'License :: OSI Approved :: Apache Software License',
            'Operating System :: OS Independent',
        ],
        packages=find_packages(exclude=['test']),
        install_requires=['dagster', 'datadog==0.28.*'],
        tests_require=['mock==2.0.*'],
        zip_safe=False,
    )


if __name__ == '__main__':
    parsed, unparsed = parser.parse_known_args()
    sys.argv = [sys.argv[0]] + unparsed
    if parsed.nightly:
        _do_setup('dagster-datadog-nightly')
    else:
        _do_setup('dagster-datadog')



from dagster_datadog import datadog_resource

from dagster import ModeDefinition, execute_solid, solid
from dagster.seven import mock


@mock.patch('datadog.statsd.timing')
@mock.patch('datadog.statsd.timed')
@mock.patch('datadog.statsd.service_check')
@mock.patch('datadog.statsd.set')
@mock.patch('datadog.statsd.distribution')
@mock.patch('datadog.statsd.histogram')
@mock.patch('datadog.statsd.decrement')
@mock.patch('datadog.statsd.increment')
@mock.patch('datadog.statsd.gauge')
@mock.patch('datadog.statsd.event')
def test_datadog_resource(
    event,
    gauge,
    increment,
    decrement,
    histogram,
    distribution,
    statsd_set,
    service_check,
    timed,
    timing,
):
    @solid
    def datadog_solid(context):
        assert context.resources.datadog

        # event
        context.resources.datadog.event('Man down!', 'This server needs assistance.')
        event.assert_called_with('Man down!', 'This server needs assistance.')

        # gauge
        context.resources.datadog.gauge('users.online', 1001, tags=["protocol:http"])
        gauge.assert_called_with('users.online', 1001, tags=["protocol:http"])

        # increment
        context.resources.datadog.increment('page.views')
        increment.assert_called_with('page.views')

        # decrement
        context.resources.datadog.decrement('page.views')
        decrement.assert_called_with('page.views')

        context.resources.datadog.histogram('album.photo.count', 26, tags=["gender:female"])
        histogram.assert_called_with('album.photo.count', 26, tags=["gender:female"])

        context.resources.datadog.distribution('album.photo.count', 26, tags=["color:blue"])
        distribution.assert_called_with('album.photo.count', 26, tags=["color:blue"])

        context.resources.datadog.set('visitors.uniques', 999, tags=["browser:ie"])
        statsd_set.assert_called_with('visitors.uniques', 999, tags=["browser:ie"])

        context.resources.datadog.service_check('svc.check_name', context.resources.datadog.WARNING)
        service_check.assert_called_with('svc.check_name', context.resources.datadog.WARNING)

        context.resources.datadog.timing("query.response.time", 1234)
        timing.assert_called_with("query.response.time", 1234)

        @context.resources.datadog.timed('run_fn')
        def run_fn():
            pass

        run_fn()
        timed.assert_called_with('run_fn')

    result = execute_solid(
        datadog_solid,
        environment_dict={
            'resources': {'datadog': {'config': {'api_key': 'NOT_USED', 'app_key': 'NOT_USED'}}}
        },
        mode_def=ModeDefinition(resource_defs={'datadog': datadog_resource}),
    )
    assert result.success
